@@ -29,7 +29,7 @@ package DBD::File;
 
 use vars qw(@ISA $VERSION $drh $valid_attrs);
 
-$VERSION = '0.33';
+$VERSION = '0.35';
 
 $drh = undef;		# holds driver handle(s) once initialised
 
@@ -381,6 +381,7 @@ sub list_tables ($) {
 
 sub quote ($$;$) {
     my($self, $str, $type) = @_;
+    if (!defined($str)) { return "NULL" }
     if (defined($type)  &&
 	($type == DBI::SQL_NUMERIC()   ||
 	 $type == DBI::SQL_DECIMAL()   ||
@@ -389,10 +390,9 @@ sub quote ($$;$) {
 	 $type == DBI::SQL_FLOAT()     ||
 	 $type == DBI::SQL_REAL()      ||
 	 $type == DBI::SQL_DOUBLE()    ||
-	 $type == DBI::TINYINT())) {
+	 $type == DBI::SQL_TINYINT())) {
 	return $str;
     }
-    if (!defined($str)) { return "NULL" }
     $str =~ s/\\/\\\\/sg;
     $str =~ s/\0/\\0/sg;
     $str =~ s/\'/\\\'/sg;
